using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class AbilityCooldown
{
    private GameManager gameManager;
    private Rigidbody rigid;
    private Transform castTransform;
    private CasterType casterType;
    private GameObject ability;

    private float cooldown;
    private float cooldownTimer;
    private ScriptableAbility scriptableAbility;

    public virtual void Initialize(GameManager gameManager, Rigidbody rigid, Transform castTransform, ScriptableAbility scriptableAbility)
    {
        this.gameManager = gameManager;
        this.rigid = rigid;
        this.castTransform = castTransform;
        cooldown = scriptableAbility.cooldown;
        cooldownTimer = cooldown;
        casterType = scriptableAbility.casterType;
        ability = scriptableAbility.ability;
        this.scriptableAbility = scriptableAbility;
    }

    public void UpdateCooldown()
    {
        cooldownTimer = Mathf.Clamp(cooldownTimer + Time.deltaTime, 0f, cooldown + 1f);
    }

    public bool Activate()
    {
        if (cooldownTimer >= cooldown)
        {
            cooldownTimer = 0;
            Trigger();

            return true;
        }

        return false;
    }

    private void Trigger()
    {
        int soundIndex = Random.Range(0, scriptableAbility.abilitySounds.Length);
        gameManager.AudioSource.PlayOneShot(scriptableAbility.abilitySounds[soundIndex]);

        Ability abilityTemp = Object.Instantiate(ability, castTransform.position, Quaternion.identity).GetComponent<Ability>();
        abilityTemp.Initialize(gameManager, rigid, castTransform, casterType);
    }

    public float GetCooldownFraction()
    {
        return cooldownTimer / cooldown;
    }
}
