using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Explode : Ability
{
    public int damage = 5;
    public float explodeRadius = 3f;
    public float explosionForce = 10f;
    public GameObject explodeEffect;

    private bool exploded;

    // Start is called before the first frame update
    protected override void Start()
    {
        ExplodeAbility();
        base.Start();
    }

    void OnDrawGizmos()
    {
        Gizmos.color = Color.yellow * 0.25f;
        Gizmos.DrawSphere(transform.position, explodeRadius);
    }

    private void ExplodeAbility()
    {
        if (exploded) return;

        exploded = true;

        Collider[] collidersHit = Physics.OverlapSphere(transform.position, explodeRadius);

        foreach (Collider collider in collidersHit)
        {
            Rigidbody rigid = collider.GetComponent<Rigidbody>();
            if (rigid != null) rigid.AddExplosionForce(explosionForce, transform.position, explodeRadius, 1f, ForceMode.Impulse);
        }

        Instantiate(explodeEffect, transform.position, Quaternion.identity);

        StartCoroutine(DelayedDamage());
    }

    private IEnumerator DelayedDamage()
    {
        yield return new WaitForSeconds(0.2f);

        Collider[] collidersHit = Physics.OverlapSphere(transform.position, explodeRadius);

        foreach (Collider collider in collidersHit)
        {
            if (casterType == CasterType.Player)
            {
                Enemy character = collider.GetComponent<Enemy>();
                if (character != null) character.TakeDamage(damage);
            }
            else
            {
                Player character = collider.GetComponent<Player>();
                if (character != null) character.TakeDamage(damage);
                castRigid.GetComponent<Enemy>().TakeDamage(int.MaxValue);
            }
        }

        Destroy(gameObject);
    }

    protected override IEnumerator LifetimeDestroy()
    {
        yield break;
    }
}
