using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Projectile : Ability
{
    public int damage = 5;
    public float initialSpeed = 50f;
    public float speed = 10f;
    public float lifetime = 8f;

    // Start is called before the first frame update
    protected override void Start()
    {
        base.Start();
        SetupProjectile();
        rigid.AddForce(transform.forward * initialSpeed, ForceMode.Impulse);
    }

    // Update is called once per frame
    protected override void Update()
    {
        base.Update();
        rigid.AddForce(transform.forward * Time.deltaTime * speed, ForceMode.Impulse);
    }

    protected virtual void OnTriggerEnter(Collider other)
    {
        if (other.tag == "Enemy" && casterType == CasterType.Player)
        {
            other.GetComponent<Enemy>().TakeDamage(damage);
            Destroy(gameObject);
        }
        else if (other.tag == "Player" && casterType == CasterType.Enemy)
        {
            other.GetComponent<Player>().TakeDamage(damage);
            Destroy(gameObject);
        }
    }

    protected override IEnumerator LifetimeDestroy()
    {
        yield return new WaitForSeconds(lifetime);
        Destroy(gameObject);
    }

    private void SetupProjectile()
    {
        if(casterType == CasterType.Enemy)
        {
            if (gameManager.Player == null) return;
            transform.LookAt(gameManager.Player.transform);
        }
        else
        {
            Ray ray = new Ray(Camera.main.transform.position, Camera.main.transform.forward);
            RaycastHit hit;
            if (Physics.Raycast(ray, out hit, 100f))
            {
                transform.LookAt(hit.point);
                Debug.DrawRay(transform.position, transform.forward, Color.red, 1f);
            }
            else
            {
                transform.LookAt(Camera.main.transform.position + (Camera.main.transform.forward * 100f));
                Debug.DrawRay(transform.position, transform.forward, Color.blue, 1f);
            }
        }
    }
}
