using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ProjectileExplode : Projectile
{
    public float explodeRadius = 3f;
    public float explosionForce = 10f;
    public GameObject explodeEffect;

    private bool exploded;

    protected override void Start()
    {
        base.Start();
    }

    protected override void Update()
    {
        base.Update();
    }

    protected override void OnTriggerEnter(Collider other)
    {
        Explode();
    }

    protected override IEnumerator LifetimeDestroy()
    {
        yield return new WaitForSeconds(lifetime);
        Explode();
    }

    private void Explode()
    {
        if (exploded) return;

        exploded = true;

        Collider[] collidersHit = Physics.OverlapSphere(transform.position, explodeRadius);

        foreach (Collider collider in collidersHit)
        {
            Rigidbody rigid = collider.GetComponent<Rigidbody>();
            if (rigid != null) rigid.AddExplosionForce(explosionForce, transform.position, explodeRadius, 1f, ForceMode.Impulse);
        }

        Instantiate(explodeEffect, transform.position, Quaternion.identity);

        GetComponent<MeshRenderer>().enabled = false;
        GetComponent<TrailRenderer>().enabled = false;
        GetComponent<Rigidbody>().isKinematic = true;

        StartCoroutine(DelayedDamage());
    }

    private IEnumerator DelayedDamage()
    {
        yield return new WaitForSeconds(0.2f);

        Collider[] collidersHit = Physics.OverlapSphere(transform.position, explodeRadius);

        foreach (Collider collider in collidersHit)
        {
            if (casterType == CasterType.Player && collider.tag == "Enemy")
            {
                collider.GetComponent<Enemy>().TakeDamage(damage);
            }
            else if (casterType == CasterType.Enemy && collider.tag == "Player")
            {
                collider.GetComponent<Player>().TakeDamage(damage);
            }
        }

        Destroy(gameObject);
    }
}
