using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Rigidbody))]
[RequireComponent(typeof(Collider))]
public abstract class Character : MonoBehaviour
{
    public int maxHealth = 10;
    public AudioClip hitSound;
    public AudioClip deathSound;

    protected GameManager gameManager;
    protected Rigidbody rigid;
    [SerializeField]
    protected int curHealth;
    protected bool isDead;

    // Start is called before the first frame update
    protected virtual void Awake()
    {
        gameManager = FindObjectOfType<GameManager>();
        rigid = GetComponent<Rigidbody>();
        curHealth = maxHealth;
    }

    // Update is called once per frame
    protected virtual void Update()
    {
        if (isDead) return;
    }
    
    public virtual void TakeDamage(int damage)
    {
        curHealth = Mathf.Clamp(curHealth - damage, 0, maxHealth);

        if(curHealth <= 0)
        {
            gameManager.AudioSource.PlayOneShot(deathSound);
            Die();
        }
        else
        {
            gameManager.AudioSource.PlayOneShot(hitSound);
        }
    }

    protected abstract void Die();

    public float GetHealthFraction()
    {
        return (float)curHealth / (float)maxHealth;
    }
}
