using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(Rigidbody))]
[RequireComponent(typeof(Collider))]
public abstract class EnemyAI : MonoBehaviour
{
    public float moveSpeed = 10f;
    public float verticalOffset = 1f;
    public float stoppingDistance = 8f;

    public ScriptableAbility scriptableAbility;

    protected AbilityCooldown abilityCooldown;
    protected GameManager gameManager;
    protected Rigidbody rigid;

    // Start is called before the first frame update
    protected virtual void Start()
    {
        gameManager = FindObjectOfType<GameManager>();
        rigid = GetComponent<Rigidbody>();
        abilityCooldown = new AbilityCooldown();
        abilityCooldown.Initialize(gameManager, rigid, transform, scriptableAbility);
    }

    // Update is called once per frame
    protected virtual void Update()
    {
        if (gameManager.Player == null) return;

        abilityCooldown.UpdateCooldown();

        transform.LookAt(gameManager.Player.transform.position + new Vector3(0, verticalOffset, 0));

        if (Vector3.Distance(transform.position, gameManager.Player.transform.position) > stoppingDistance)
        {
            rigid.velocity = (gameManager.Player.transform.position - transform.position).normalized * moveSpeed * Time.deltaTime;
        }
        else
        {
            rigid.velocity = Vector3.zero;
            Attack();
        }
        Debug.DrawLine(transform.position, transform.position + (transform.forward * stoppingDistance), Color.magenta);
    }

    protected abstract void Attack();
}

