using Invector.vCharacterController;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class Player : Character
{
    public int maxEssence = 100;
    public ScriptableAbility primaryAbilityScriptable;
    public ScriptableAbility secondaryAbilityScriptable;
    public ScriptableAbility ultimateAbilityScriptable;
    public Transform castTransform;
    public GameObject deathEffect;

    public AbilityCooldown primaryAbilityCooldown;
    public AbilityCooldown secondaryAbilityCooldown;
    public AbilityCooldown ultimateAbilityCooldown;
    [SerializeField]
    private int curEssence = 0;

    protected override void Awake()
    {
        base.Awake();

        InitializeAbilities();
    }

    protected override void Update()
    {
        base.Update();
        HandleAbilityInput();
        UpdateCooldowns();
    }

    private void HandleAbilityInput()
    {
        //Primary input
        if(Input.GetMouseButtonDown(0))
        {
            primaryAbilityCooldown.Activate();
        }

        //Secondary input
        if (Input.GetMouseButtonDown(1))
        {
            secondaryAbilityCooldown.Activate();
        }

        //Ultimate input
        if (Input.GetKeyDown(KeyCode.F))
        {
            ultimateAbilityCooldown.Activate();
        }
    }

    private void InitializeAbilities()
    {
        primaryAbilityCooldown = new AbilityCooldown();
        secondaryAbilityCooldown = new AbilityCooldown();
        ultimateAbilityCooldown = new AbilityCooldown();
        primaryAbilityCooldown.Initialize(gameManager, rigid, castTransform, primaryAbilityScriptable);
        secondaryAbilityCooldown.Initialize(gameManager, rigid, castTransform, secondaryAbilityScriptable);
        ultimateAbilityCooldown.Initialize(gameManager, rigid, castTransform, ultimateAbilityScriptable);
    }

    private void UpdateCooldowns()
    {
        primaryAbilityCooldown.UpdateCooldown();
        secondaryAbilityCooldown.UpdateCooldown();
        ultimateAbilityCooldown.UpdateCooldown();
    }

    private void OnTriggerEnter(Collider other)
    {
        if (other.gameObject.tag == "Essence")
        {
            Essence ess = other.GetComponent<Essence>();

            if (ess.Target != transform) return;

            if(ess.canGive)
            {
                curEssence = Mathf.Clamp(curEssence + 1, 0, maxEssence);
                ess.canGive = false;
            }
            Destroy(other.gameObject);
        }
    }

    public bool ConsumeEssence(Transform tartget)
    {
        if (curEssence > 0)
        {
            Essence essence = Instantiate(gameManager.essencePrefab, transform.position, Quaternion.identity).GetComponent<Essence>();
            essence.Setup(tartget.transform, 1f);
            curEssence = Mathf.Clamp(curEssence - 1, 0, maxEssence);
            return true;
        }

        return false;
    }

    public void StopPlayer()
    {
        isDead = true;
        GetComponent<vThirdPersonController>().enabled = false;
        GetComponent<vThirdPersonInput>().enabled = false;
        Rigidbody rigid = GetComponent<Rigidbody>();
        rigid.velocity = Vector3.zero;
        rigid.useGravity = false;
        rigid.isKinematic = true;
    }

    protected override void Die()
    {
        StopPlayer();
        Instantiate(deathEffect, transform.position, Quaternion.identity);
        gameManager.Lose();
    }

    public float GetEssenceFraction()
    {
        return (float)curEssence / (float)maxEssence;
    }

    public float[] GetAbilityFractions()
    {
        return new float[]
        {
            primaryAbilityCooldown.GetCooldownFraction(),
            secondaryAbilityCooldown.GetCooldownFraction(),
            ultimateAbilityCooldown.GetCooldownFraction()
        };
    }
}
