using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class EnemySpawnManager : MonoBehaviour
{
    public float verticalOffset = 5f;
    public float randomOffset = 3f;

    private GameManager gameManager;
    private ProgressionManager progressionManager;

    private List<Enemy> enemies;
    bool spawning = true;

    // Start is called before the first frame update
    void Start()
    {
        gameManager = GetComponent<GameManager>();
        progressionManager = GetComponent<ProgressionManager>();
        enemies = new List<Enemy>();
    }

    // Update is called once per frame
    void Update()
    {
        
    }

    public void StartLevels()
    {
        StartCoroutine(progressionManager.WaitForNextLevel());
        StartCoroutine(ProgressiveEnemySpawn());
    }

    public IEnumerator ProgressiveEnemySpawn()
    {
        while(true)
        {
            yield return new WaitForSeconds(progressionManager.GetSpawnRate());

            SpawnEnemy();
        }
    }

    private void SpawnEnemy()
    {
        if (gameManager.Player == null || !spawning) return;

        Vector3 initialPoint = gameManager.Player.transform.position + new Vector3(0, verticalOffset, 0);
        Vector3 targetPoint;

        while (true)
        {
            float randomAngle = Random.Range(0f, 360f);
            targetPoint = initialPoint + (Quaternion.Euler(0, randomAngle, 0) * Vector3.forward * progressionManager.GetSpawnDistance());
            Debug.DrawRay(targetPoint, Vector3.up, Color.green, 10f);

            targetPoint += new Vector3(0, Random.Range(-randomOffset, randomOffset), 0);

            Collider[] collidersHit = Physics.OverlapSphere(targetPoint, 2);
            if (collidersHit.Length <= 0) break;
        }

        Enemy enemy = Instantiate(progressionManager.SelectEnemy(), targetPoint, Quaternion.identity).GetComponent<Enemy>();
        enemies.Add(enemy);
    }

    public void KillAllEnemies()
    {
        for (int i = enemies.Count - 1; i >=  0; i--)
        {
            Destroy(enemies[i]);
            enemies.RemoveAt(i);
        }

        spawning = false;
    }

    public void RemoveEnemy(Enemy enemy)
    {
        if (enemies.Contains(enemy)) enemies.Remove(enemy);
    }
}
