using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

[RequireComponent(typeof(HUDManager))]
[RequireComponent(typeof(TileManager))]
[RequireComponent(typeof(EnemySpawnManager))]
[RequireComponent(typeof(AudioSource))]
public class GameManager : MonoBehaviour
{
    public int reqEssence = 50;
    public int curEssence = 0;
    public GameObject playerPrefab;
    public GameObject playerCamera;
    public GameObject essencePrefab;

    private TileManager tileManager;

    private Player playerCache;
    public Player Player
    {
        get
        {
            if (playerCache == null)
            {
                GameObject playerObj = GameObject.FindGameObjectWithTag("Player");
                if (playerObj != null) playerCache = playerObj.GetComponent<Player>();
                else return null;
            }
            return playerCache;
        }
    }

    private AudioSource audioSource;
    public AudioSource AudioSource
    {
        get
        {
            if (audioSource == null)
            {
                audioSource = GetComponent<AudioSource>();
            }
            return audioSource;
        }
    }

    // Start is called before the first frame update
    void Awake()
    {
        tileManager = GetComponent<TileManager>();
    }

    private void Start()
    {

    }

    // Update is called once per frame
    void Update()
    {
        
    }

    public void StartGame()
    {
        Destroy(GameObject.Find("Camera"));
        Instantiate(playerPrefab, tileManager.initialShrine.spawnPoint.position, Quaternion.identity).GetComponent<Player>();
        Instantiate(playerCamera);
        GetComponent<EnemySpawnManager>().StartLevels();
    }

    public void Win()
    {
        Player.StopPlayer();
        GetComponent<EnemySpawnManager>().KillAllEnemies();
        GetComponent<HUDManager>().StartWin();
        StartCoroutine(ReloadScene());
    }

    public void Lose()
    {
        GetComponent<EnemySpawnManager>().KillAllEnemies();
        GetComponent<HUDManager>().StartLoss();
        StartCoroutine(ReloadScene());
    }

    private IEnumerator ReloadScene()
    {
        yield return new WaitForSeconds(5f);
        Cursor.visible = true;
        Cursor.lockState = CursorLockMode.Confined;
        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
    }

    public void IncrementEssence(int essence = 1)
    {
        curEssence += essence;

        if(curEssence >= reqEssence)
        {
            tileManager.LiftRandomSpawnTile();
            curEssence = 0;
        }
    }
}
