using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class HUDManager : MonoBehaviour
{
    public Image healthbarImage;
    public Image essencebarImage;
    public Image abilityPrimaryImage;
    public Image abilitySecondaryImage;
    public Image abilityUltimateImage;
    public Animator animator;
    public Slider sfxSlider;
    public Slider musicSlider;

    private GameManager gameManager;
    private GameObject HUD;
    private GameObject Menu;

    // Start is called before the first frame update
    void Start()
    {
        gameManager = GetComponent<GameManager>();
        HUD = transform.GetChild(0).transform.GetChild(0).gameObject;
        Menu = transform.GetChild(0).transform.GetChild(1).gameObject;

        HUD.SetActive(false);
        Menu.SetActive(true);

        LoadSettings();
    }

    // Update is called once per frame
    void Update()
    {
        if (gameManager.Player != null)
        {
            float[] abilityFills = gameManager.Player.GetAbilityFractions();
            healthbarImage.fillAmount = gameManager.Player.GetHealthFraction();
            essencebarImage.fillAmount = gameManager.Player.GetEssenceFraction();
            abilityPrimaryImage.fillAmount = abilityFills[0];
            abilitySecondaryImage.fillAmount = abilityFills[1];
            abilityUltimateImage.fillAmount = abilityFills[2];
        }
    }

    public void StartGame()
    {
        HUD.SetActive(true);
        Menu.SetActive(false);
        GetComponent<GameManager>().StartGame();
        Cursor.visible = false;
        Cursor.lockState = CursorLockMode.Locked;
    }

    public void ExitGame()
    {
        Application.Quit();
    }

    public void StartWin()
    {
        animator.SetTrigger("Win");
    }

    public void StartLoss()
    {
        animator.SetTrigger("Lose");
    }

    private void LoadSettings()
    {
        float sfx = PlayerPrefs.GetFloat("sfx", 1f);
        sfxSlider.value = sfx;
        float music = PlayerPrefs.GetFloat("music", 0.25f);
        musicSlider.value = music;
        UpdateVolumes();
    }

    public void SaveMusicLevel(float val)
    {
        PlayerPrefs.SetFloat("music", val);
        UpdateVolumes();
    }

    public void SaveSFXLevel(float val)
    {
        PlayerPrefs.SetFloat("sfx", val);
        UpdateVolumes();
    }

    private void UpdateVolumes()
    {
        gameManager.AudioSource.volume = sfxSlider.value;
        GameObject.FindGameObjectWithTag("Music").GetComponent<AudioSource>().volume = musicSlider.value;
    }
}
