using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Rendering;
using UnityEngine.Rendering.HighDefinition;

public class ProgressionManager : MonoBehaviour
{
    public Level[] levels;
    public float lerpDuration = 3f;

    private Renderer clouds;
    private int curLevel = 0;

    // Start is called before the first frame update
    void Start()
    {
        curLevel = 0;
        clouds = GameObject.Find("Clouds").GetComponent<Renderer>();
        
        UpdateAtmosphere();
    }

    private void Update()
    {
    }

    public IEnumerator WaitForNextLevel()
    {
        yield return new WaitForSeconds(levels[curLevel].duration);
        TransitionLevel();
    }

    private void TransitionLevel()
    {
        curLevel = Mathf.Clamp(curLevel + 1, 0, levels.Length - 1);
        UpdateAtmosphere();

        StartCoroutine(WaitForNextLevel());
    }

    private void UpdateAtmosphere()
    {
        Color[] lerpColors = levels[curLevel].GetAtmosphereColors();

        clouds.material.SetColor("_lightcolor", lerpColors[0]);
        clouds.material.SetColor("_darkcolor", lerpColors[1]);

        foreach (Level level in levels)
        {
            level.volume.SetActive(false);
        }
        levels[curLevel].volume.SetActive(true);
    }

    public float GetSpawnRate()
    {
        return levels[curLevel].spawnRate;
    }

    public float GetSpawnDistance()
    {
        return levels[curLevel].spawnDistance;
    }

    public GameObject SelectEnemy()
    {
        return levels[curLevel].SelectEnemy();
    }

    [System.Serializable]
    public class Level
    {
        public float duration = 20f;
        public float spawnRate = 2f;
        public float spawnDistance = 50f;
        public EnemySelector[] enemySelectors;
        public Color cloudLightColor;
        public Color cloudShadowColor;
        public GameObject volume;

        public GameObject SelectEnemy()
        {
            int rnd = Random.Range(0, TotalWeight());
            int cur = 0;
            for (int i = 0; i < enemySelectors.Length; i++)
            {
                if (rnd < cur + enemySelectors[i].weighting && rnd >= cur)
                {
                    return enemySelectors[i].enemy;
                }
                cur += enemySelectors[i].weighting;
            }

            return enemySelectors[0].enemy;
        }

        public Color[] GetAtmosphereColors()
        {
            return new Color[]
            {
                cloudLightColor,
                cloudShadowColor
            };
        }

        private int TotalWeight()
        {
            int total = 0;
            foreach (EnemySelector es in enemySelectors)
            {
                total += es.weighting;
            }
            return total;
        }
    }

    [System.Serializable]
    public class EnemySelector
    {
        public GameObject enemy;
        public int weighting = 1;
    }
}