using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Shrine : MonoBehaviour
{
    public float collectionRadius = 10f;
    public float consumptionRate = 1f;
    public int consumeAmount = 3;

    private GameManager gameManager;
    private float consumptionTimer = 0f;

    // Start is called before the first frame update
    void Awake()
    {
        gameManager = FindObjectOfType<GameManager>();
    }

    // Update is called once per frame
    void Update()
    {
        ShrineConsumptionHandler();
    }

    private void ShrineConsumptionHandler()
    {
        if(gameManager.Player == null) return;

        if(Vector3.Distance(gameManager.Player.transform.position, transform.position) <= collectionRadius)
        {
            consumptionTimer += Time.deltaTime;

            if (consumptionTimer >= consumptionRate)
            {
                for (int i = 0; i < consumeAmount; i++)
                {
                    if (gameManager.Player.ConsumeEssence(transform)) gameManager.IncrementEssence();
                }
                consumptionTimer = 0f;
            }
            return;
        }

        consumptionTimer = 0f;
    }

    private void OnTriggerEnter(Collider other)
    {
        if (other.gameObject.tag == "Essence")
        {
            Essence ess = other.GetComponent<Essence>();
            if (ess.Target != transform) return;
            Destroy(other.gameObject);
        }
    }

    void OnDrawGizmos()
    {
        Gizmos.color = Color.red * 0.25f;
        Gizmos.DrawSphere(transform.position, collectionRadius);
    }
}
