using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Tile : MonoBehaviour
{
    public bool isSpecial;
    public GameObject tileMesh;
    public float speed = 2f;
    public bool lifted;
    public int x;
    public int z;
    public TileTopSpawner tileTopSpawner;

    private Vector3 initialPosition;
    private float verticalOffset = -15;
    private bool isLifting;

    public void Initialize(int x, int z)
    {
        this.x = x;
        this.z = z;
        initialPosition = tileMesh.transform.localPosition;
        GenerateTileTop();
        tileMesh.transform.localPosition = initialPosition + new Vector3(0, verticalOffset, 0);
    }

    // Start is called before the first frame update
    void Start()
    {
    }

    // Update is called once per frame
    void Update()
    {
        if (isLifting)
        {
            if (tileMesh.transform.localPosition.y < initialPosition.y)
            {
                tileMesh.transform.localPosition = Vector3.Lerp(tileMesh.transform.localPosition, initialPosition, Time.deltaTime * speed);
            }
            else
            {
                isLifting = false;
            }
        }
    }

    public void Lift(bool instant = false)
    {
        lifted = true;

        if (instant)
        {
            tileMesh.transform.localPosition = initialPosition;
            return;
        }

        isLifting = true;
    }

    private void GenerateTileTop()
    {
        if (isSpecial)
        {
            return;
        }

        GameObject obj = Instantiate(tileTopSpawner.SelectTop(), tileMesh.transform);
        obj.transform.localPosition = Vector3.zero;
        obj.transform.rotation = Quaternion.Euler(0, GetRandomBearingRotation(), 0);
    }

    private float GetRandomBearingRotation()
    {
        Bearing bearing = (Bearing)Random.Range(0, 4);
        return GetBearingRotation(bearing);
    }

    private float GetBearingRotation(Bearing bearing)
    {
        switch (bearing)
        {
            case Bearing.North:
                return 90f;
            case Bearing.East:
                return 180f;
            case Bearing.South:
                return 270f;
            default:
                return 0f;
        }
    }

    [System.Serializable]
    public class TileTopSpawner
    {
        public TileTopSelector[] topSelectors;

        public GameObject SelectTop()
        {
            int rnd = Random.Range(0, TotalWeight());
            int cur = 0;
            for (int i = 0; i < topSelectors.Length; i++)
            {
                if (rnd < cur + topSelectors[i].weighting && rnd >= cur)
                {
                    return topSelectors[i].top;
                }
                cur += topSelectors[i].weighting;
            }

            return topSelectors[0].top;
        }

        private int TotalWeight()
        {
            int total = 0;
            foreach (TileTopSelector tts in topSelectors)
            {
                total += tts.weighting;
            }
            return total;
        }
    }

    [System.Serializable]
    public class TileTopSelector
    {
        public GameObject top;
        public int weighting = 1;
    }

    private enum Bearing
    {
        North,
        East,
        South,
        West
    }

}
